/************************************************************/
/*                                                          */
/* THIS IS A DEMO PROGRAM FOR USING A DMA CHANNEL TO        */
/* COPY DATA FROM A MEMORY BUFFER INTO ANOTHER BUFFER       */
/*                                                          */
/*     PW    ver 1.0    20-12-02   first written, taken from */
/*                                        sorting example    */
/************************************************************/

#include <stdio.h>
#include <std.h>

/* Use HERON-API for heron1 so you need the next include line*/
#include "heron2.h"

 
/* defines used specifically in this program*/
#define BUFFER_SIZE     1024   /* number of samples per chan   */

    
int copyofdata[BUFFER_SIZE];


volatile unsigned int *mydma_pri_ctrl;
volatile unsigned int *mydma_sec_ctrl;
volatile unsigned int *mydma_src_addr;
volatile unsigned int *mydma_dst_addr;
volatile unsigned int *mydma_trn_cntr;

                             
void dma_start_copy(int dma_number, unsigned int from, unsigned int to, int n)
{


switch (dma_number)
{
case 0: mydma_pri_ctrl =(volatile unsigned int *)0x01840000;
	    mydma_sec_ctrl =(volatile unsigned int *)0x01840008;
		mydma_src_addr =(volatile unsigned int *)0x01840010;
		mydma_dst_addr =(volatile unsigned int *)0x01840018;
		mydma_trn_cntr =(volatile unsigned int *)0x01840020;
		break;
case 1: mydma_pri_ctrl =(volatile unsigned int *)0x01840040;
	    mydma_sec_ctrl =(volatile unsigned int *)0x01840048;
		mydma_src_addr =(volatile unsigned int *)0x01840050;
		mydma_dst_addr =(volatile unsigned int *)0x01840058;
		mydma_trn_cntr =(volatile unsigned int *)0x01840060;
		break;	
case 2: mydma_pri_ctrl =(volatile unsigned int *)0x01840004;
	    mydma_sec_ctrl =(volatile unsigned int *)0x0184000C;
		mydma_src_addr =(volatile unsigned int *)0x01840014;
		mydma_dst_addr =(volatile unsigned int *)0x0184001C;
		mydma_trn_cntr =(volatile unsigned int *)0x01840024;
		break;		
case 3: mydma_pri_ctrl =(volatile unsigned int *)0x01840044;
	    mydma_sec_ctrl =(volatile unsigned int *)0x0184004C;
		mydma_src_addr =(volatile unsigned int *)0x01840054;
		mydma_dst_addr =(volatile unsigned int *)0x0184005C;
		mydma_trn_cntr =(volatile unsigned int *)0x01840064;
		break;        
		}

   *mydma_pri_ctrl     = 0;                /* Stop/initialise DMA */

   *mydma_src_addr     = from;
   *mydma_dst_addr 	   = to;
   *mydma_trn_cntr     = n;
   *mydma_sec_ctrl     = 0x00000000;
   *mydma_pri_ctrl     = 0x00000051;           /* start dma */

}


void dma_wait_end(int dma_number)
{

switch (dma_number)
{
case 0: mydma_pri_ctrl =(volatile unsigned int *)0x01840000;
		break;
case 1: mydma_pri_ctrl =(volatile unsigned int *)0x01840040;
		break;	
case 2: mydma_pri_ctrl =(volatile unsigned int *)0x01840004;
		break;		
case 3: mydma_pri_ctrl =(volatile unsigned int *)0x01840044;
		break;        
		}
   while ((*mydma_pri_ctrl&0xc)!=0) 
   {
	asm("   NOP 5"); /* these make it interruptable*/
	asm("   NOP 5");
	asm("   NOP 5");
	asm("   NOP 5");
	asm("   NOP 5");
   }
}        
              
                       
               
main()
{
}        
        
       

maintask (void)
{
/* WARNING the default stack size for this task ( maintask) defined in    */
/* the cdb file is 8192 bytes                                             */
/* This means that you cannot statically declare more than 2048 integers  */
/* in this task.                                                          */
/* You can increase the stack size allowed by changing the settings for   */
/* the task in the cdb file.                                              */
/* If you need more than 2048 integers, it is better to use Malloc/Free   */
/* to allocate your data buffers 					  */

int status;
int i,dma_channel;               
int *databuf;
          

printf("Hello from DMA copying demo.\n");

    /* RELEASE THE CONFIG LINE */
    HeronConfigOff(); 
    
/*get a DMA engine to use*/
dma_channel=HeronDmaClaim();
if (dma_channel==-1) {
	printf("Cannot get DMA from HERON-API \n");
	exit(0);
}                 


/* this buffer will be in external memory because we have used malloc*/
databuf=(int *)malloc(BUFFER_SIZE*sizeof(int));
if (databuf==NULL) {
	printf("Error failed to malloc space \n");
	exit(0);
}


    /* SET UP THE DATA BUFFERS */
for (i=0;i<BUFFER_SIZE;i++)    
{            
    databuf[i]=i;
    copyofdata[i]=0xc0ffee;	
}
				
   
dma_start_copy(dma_channel,(unsigned int)databuf, (unsigned int)copyofdata, BUFFER_SIZE);
	     	    
/*       which will start the dma to copy the data from external memory */
/*       buffer dtaabuf into the internal memory bufer copyofdata       */

/* we could be processing soemthing with the CPU here while the data is being copied*/

/* now wait until the dma is complete*/

dma_wait_end(dma_channel);


/* check the data has been copied */
for (i=0;i<BUFFER_SIZE;i++)    
{            
    if (copyofdata[i]!=i)printf("erorr found %x at %x\n",copyofdata[i],i);
}
printf("finished checking\n");
                                                 
status =HeronDmaFree(dma_channel);
if (status!=HERON_OK) {
	printf("error freeing DMA no %d to HERON-API \n",dma_channel);
	exit(0);
}


} /* end maintask */
