/************************************************************/
/*                                                          */
/* THIS IS A DEMO PROGRAM FOR USING A DMA CHANNEL TO        */
/* UNPACK DATA FROM A MEMORY BUFFER INTO SEPERATE BUFFERS   */
/*                                                          */
/*     PW    ver 1.0    23-6-00   first written, taken from */
/*                                               GD9 example*/
/*     PW    ver 2.0	14-7-00   changed for CCS 1.2       */
/************************************************************/

#include <stdio.h>
#include <std.h>

/* Use HERON-API for heron1 so you need the next include line*/
#include "heron1.h"

 
/* defines used specifically in this program*/
#define SAMPLE_SIZE     128   /* number of samples per chan   */
#define NO_OF_CHANNELS 4   /* less than 4 because we have only defined thatmany buffers*/
#define BUF_SIZE (SAMPLE_SIZE*NO_OF_CHANNELS)
    

short     ch0_data[SAMPLE_SIZE];
short     ch1_data[SAMPLE_SIZE];
short     ch2_data[SAMPLE_SIZE];
short     ch3_data[SAMPLE_SIZE];

short   bufx[BUF_SIZE]; 
short   dest[BUF_SIZE];  

volatile unsigned int *mydma_pri_ctrl;
volatile unsigned int *mydma_sec_ctrl;
volatile unsigned int *mydma_src_addr;
volatile unsigned int *mydma_dst_addr;
volatile unsigned int *mydma_trn_cntr;

volatile unsigned int *mydma_glb_cnt_rld_a ;
volatile unsigned int *mydma_glb_cnt_rld_b ;

volatile unsigned int *mydma_glb_idx_a;
volatile unsigned int *mydma_glb_idx_b;
                             
/********************************************************/
/*							*/                             
/*   void setup_dma_addresses(int dma_number)           */
/* sets up the correct pointers to the dma channel      */
/* depending which one we have been given               */
/********************************************************/
      
void setup_dma_addresses(int dma_number)
{
switch (dma_number)
{
case 0: mydma_pri_ctrl =(volatile unsigned int *)0x01840000;
	    mydma_sec_ctrl =(volatile unsigned int *)0x01840008;
		mydma_src_addr =(volatile unsigned int *)0x01840010;
		mydma_dst_addr =(volatile unsigned int *)0x01840018;
		mydma_trn_cntr =(volatile unsigned int *)0x01840020;
		break;
case 1: mydma_pri_ctrl =(volatile unsigned int *)0x01840040;
	    mydma_sec_ctrl =(volatile unsigned int *)0x01840048;
		mydma_src_addr =(volatile unsigned int *)0x01840050;
		mydma_dst_addr =(volatile unsigned int *)0x01840058;
		mydma_trn_cntr =(volatile unsigned int *)0x01840060;
		break;	
case 2: mydma_pri_ctrl =(volatile unsigned int *)0x01840004;
	    mydma_sec_ctrl =(volatile unsigned int *)0x0184000C;
		mydma_src_addr =(volatile unsigned int *)0x01840014;
		mydma_dst_addr =(volatile unsigned int *)0x0184001C;
		mydma_trn_cntr =(volatile unsigned int *)0x01840024;
		break;		
case 3: mydma_pri_ctrl =(volatile unsigned int *)0x01840044;
	    mydma_sec_ctrl =(volatile unsigned int *)0x0184004C;
		mydma_src_addr =(volatile unsigned int *)0x01840054;
		mydma_dst_addr =(volatile unsigned int *)0x0184005C;
		mydma_trn_cntr =(volatile unsigned int *)0x01840064;
		break;        
		}
    
mydma_glb_cnt_rld_a = (volatile unsigned int *)0x01840028;
mydma_glb_cnt_rld_b = (volatile unsigned int *)0x0184002C;

mydma_glb_idx_a = (volatile unsigned int *)0x01840030;
mydma_glb_idx_b = (volatile unsigned int *)0x01840034;    
}                             


/********************************************************/
/*														*/
/* dma_sort_16(int dma_number,int n_channels, short *src, short *dest, int length)     */
/*														*/
/*	  sorts multi-channel 16 bit data such as from an   */
/*    A/D using DMA.                                    */
/*	  length is numner of 16 bit elements													*/
/********************************************************/

void dma_sort_16(int dma_number,int n_channels, short *src, short *destin, int length)     
{                                                 
         
*mydma_pri_ctrl = 0;
*mydma_src_addr = (unsigned int)src;
*mydma_dst_addr = (unsigned int)destin; 
*mydma_trn_cntr = (n_channels<<16)|length/n_channels;
*mydma_sec_ctrl= 0;
*mydma_glb_cnt_rld_a = length/n_channels;

/* Bit 13 of primary control register if set low selects global index register A */
/* Bit 13 of primary control register if set high selects global index register B */
/* Bits 4 and 5 control src address modification. Set to 0x11 to step by index value */
/* in the selected global index register A */
/* Bits 6 and 7 control dst address modification. Set to 0x11 to step by index value */
/* in the selected global index register A */


/* To step through dst buffer by one word at a time but to step through the source in n, */
/* where n is a signed 16-bit value, */
*mydma_glb_idx_a = -((length-n_channels-1)*2)<<16 | n_channels*2;
*mydma_pri_ctrl = 0x01000171; 

/* Wait for it to complete while being interruptable*/
while (((*mydma_pri_ctrl) & 0xc) != 0) {
	asm("   NOP 5");
	asm("   NOP 5");
	asm("   NOP 5");
	asm("   NOP 5");
	asm("   NOP 5");
	asm("   NOP 5");
}
	
return;	 
}

                       
               
main()
{
}        
        
       

maintask (void)
{
/* WARNING the default stack size for this task ( maintask) defined in    */
/* the cdb file is 8192 bytes                                             */
/* This means that you cannot statically declare more than 2048 integers  */
/* in this task.                                                          */
/* You can increase the stack size allowed by changing the settings for   */
/* the task in the cdb file.                                              */
/* If you need more than 2048 integers, it is better to use Malloc/Free   */
/* to allocate your data buffers 					  */

int status;
int i,j,dma_channel;               

          

printf("Hello from DMA sorting demo.\n");

    /* RELEASE THE CONFIG LINE */
    HeronConfigOff(); 
    
/*get a DMA engine to use*/
dma_channel=HeronDmaClaim();
if (dma_channel==-1) {
	printf("Cannot get DMA from HERON-API \n");
	exit(0);
}                 

setup_dma_addresses(dma_channel);

    /* SET UP DATA THAT NEEDS SORTING */
    for (i=0;i<BUF_SIZE;)    
    {       
    for(j=0;j<NO_OF_CHANNELS;j++)
    {            
    bufx[i]=j;
    i++;
    }
				
   	}     

	     dma_sort_16(dma_channel,NO_OF_CHANNELS,bufx, dest, BUF_SIZE);
	     	    
/*       which will copy the data from the source array bufx,   */
/*       into the array "dest[]", but separating it into the    */
/*       same number of sections as there are channels. Then the*/
/*       data for channel 1 is in the first section, the data   */
/*       for the next channel comes after etc etc..             */
                     	
                                                      

status =HeronDmaFree(dma_channel);
if (status!=HERON_OK) {
	printf("error freeing DMA no %d to HERON-API \n",dma_channel);
	exit(0);
}


} /* end maintask */
